<?php

namespace Study\Course\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Study\Course\Forms\Course as CourseForm;
use Study\Course\Http\Requests\CourseRequest;
use Study\Course\Interfaces\CourseRepositoryInterface;
use Study\Course\Repositories\Eloquent\Filters\CourseResourceFilter;
use Study\Course\Repositories\Eloquent\Presenters\CourseListPresenter;

/**
 * Resource controller class for course.
 */
class CourseResourceController extends BaseController
{

    /**
     * Initialize course resource controller.
     *
     *
     * @return null
     */
    public function __construct(CourseRepositoryInterface $course)
    {
        parent::__construct();
        $this->form = CourseForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('study.course.modules'), 'course', guard_url('course'));
        $this->repository = $course;
    }

    /**
     * Display a list of course.
     *
     * @return Response
     */
    public function index(CourseRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CourseResourceFilter::class)
            ->setPresenter(CourseListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('course::course.names'))
            ->view('course::course.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display course.
     *
     * @param Request $request
     * @param Model   $course
     *
     * @return Response
     */
    public function show(CourseRequest $request, CourseRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('course::course.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('course::course.show')
            ->output();
    }

    /**
     * Show the form for creating a new course.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CourseRequest $request, CourseRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('course::course.name'))
            ->view('course::course.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new course.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CourseRequest $request, CourseRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('course::course.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('course/course/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/course/course'))
                ->redirect();
        }

    }

    /**
     * Show course for editing.
     *
     * @param Request $request
     * @param Model   $course
     *
     * @return Response
     */
    public function edit(CourseRequest $request, CourseRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('course::course.name'))
            ->view('course::course.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the course.
     *
     * @param Request $request
     * @param Model   $course
     *
     * @return Response
     */
    public function update(CourseRequest $request, CourseRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('course::course.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('course/course/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('course/course/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the course.
     *
     * @param Model   $course
     *
     * @return Response
     */
    public function destroy(CourseRequest $request, CourseRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('course::course.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('course/course/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('course/course/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
